#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Frontend API
"""
from sense.client.siterm.base_api import BaseApi

class FrontendApi(BaseApi):
    """Frontend API for SENSE-SiteRM"""
    def __init__(self):
        super().__init__()

    def get_alive(self, **kwargs):
        """Get alive status from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/alive",
                                       **{"verb": "GET", "data": {}})

    def get_ready(self, **kwargs):
        """Get ready status from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/ready",
                                       **{"verb": "GET", "data": {}})

    def get_liveness(self, **kwargs):
        """Get liveness status from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/liveness",
                                       **{"verb": "GET", "data": {}})

    def get_readiness(self, **kwargs):
        """Get readiness status from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/readiness",
                                       **{"verb": "GET", "data": {}})

    def get_sitename(self, **kwargs):
        """Get sitename from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/frontend/sitename",
                                       **{"verb": "GET", "data": {}})

    def get_configuration(self, **kwargs):
        """Get configuration from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/frontend/configuration",
                                       **{"verb": "GET", "data": {}})

    def get_switchdata(self, **kwargs):
        """Get switch data from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/frontend/getswitchdata",
                                       **{"verb": "GET", "data": {}})

    def get_activedeltas(self, **kwargs):
        """Get active deltas from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url="/api/frontend/activedeltas",
                                       **{"verb": "GET", "data": {}})

    def get_qosdata(self, **kwargs):
        """Get QoS data from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/frontend/qosdata",
                                       **{"verb": "GET", "data": {}})

    def get_topology(self, **kwargs):
        """Get topology from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/topo/gettopology",
                                       **{"verb": "GET", "data": {}})

    def get_hostdata(self, **kwargs):
        """Get host data from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/hosts",
                                       **{"verb": "GET", "data": {}})



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Services API
"""
from sense.client.siterm.base_api import BaseApi

class ServicesApi(BaseApi):
    """Services API for SENSE-SiteRM"""
    def __init__(self):
        super().__init__()

    def get_services(self, **kwargs):
        """Get services from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['hostname', 'servicename']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/services",
                                       **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def get_servicestates(self, **kwargs):
        """Get service states from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['limit']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/servicesstates",
                                       **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def reloadconfig(self, **kwargs):
        """Reload configuration via SENSE-SiteRM API for specific entry"""
        sitename = self.getSitename(**kwargs)
        vals = {"hostname": "default", "servicename": "ALL",
                "sitename": sitename, "action": "reload"}
        for key in vals:
            if key in kwargs and kwargs[key]:
                vals[key] = kwargs[key]
            if not vals.get(key):
                raise Exception(f"Key {key} is required for reloadconfig")
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/serviceaction",
                                       **{"verb": "POST", "data": vals})



#!/usr/bin/env python3
# coding: utf-8
# pylint: disable=too-few-public-methods,no-member
"""
    SENSE-SiteRM Base Class for All APIs
"""
from sense.client.siterm.requestwrapper import RequestWrapper

class BaseApi:
    """Base class for all SENSE-SiteRM APIs"""
    def __init__(self):
        self.client = RequestWrapper()

    def getSitename(self, **kwargs):
        """Get sitename from kwargs or urn"""
        sitename = kwargs.get("sitename", None)
        urn = kwargs.get("urn", None)
        if sitename and urn:
            # Check if urn sitename matches sitename
            sitenameFromUrn = self.client.getSitenameFromUrn(urn)
            if sitenameFromUrn != sitename:
                raise Exception(f"Urn {urn} does not match sitename {sitename} (urn should be {sitenameFromUrn})")
        if not sitename and urn:
            sitename = self.client.getSitenameFromUrn(urn)
        if not sitename:
            raise Exception("Sitename is required for API calls (or urn to get sitename dynamically)")
        return sitename



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Debug API
"""
from sense.client.siterm.base_api import BaseApi


class DebugApi(BaseApi):
    """Debug API for SENSE-SiteRM"""
    def __init__(self):
        super().__init__()

    # TODO. Need to rewrite with checked Frontend output from debugactioninfo
    # Each Frontend and Site can support different parameters and actions. And also have different defaults.
    #def _validate_input(self, keys, kwargs, action):
    #    """Validate input for debug actions"""
    #    if set(kwargs.keys()) != set(keys):
    #        raise Exception(f"Wrong input for {action}. Must have these keys: {keys}. Input: {kwargs}")


    def get_debugactions(self, **kwargs):
        """Get debug actions from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debugactions",
                                       **{"verb": "GET", "data": {}})

    def get_debugactioninfo(self, **kwargs):
        """Get debug action info from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        action = kwargs.get("action", None)
        if not action:
            raise Exception("Action is required for get_debugactioninfo")
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debugactioninfo",
                                       **{"verb": "GET", "data": {}, "urlparams": {"action": action}})


    def __submit_call(self, **kwargs):
        """Submit a debug call to the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        submitout = {"hostname": kwargs.get("hostname", "undefined"),
                     "request": kwargs}
        return self.client.makeRequest(sitename=sitename, url=f"/api/{sitename}/debug",
                                       **{"verb": "POST", "data": submitout})

    def _check_action_supported(self, action, **kwargs):
        """Check if the action is supported by the SENSE-SiteRM instance"""
        actions = self.get_debugactions(**kwargs)
        if action not in actions[0].get("actions", []):
            raise Exception(f"Action {action} is not supported by the {kwargs.get('sitename', None)} instance.")

    def submit(self, **kwargs):
        """Submit a debug action to the SENSE-SiteRM API"""
        action = kwargs.get("action", None)
        if not action:
            raise Exception("Action is required for submit")
        self._check_action_supported(action, **kwargs)
        return self.__submit_call(**kwargs)

    def submit_ping(self, **kwargs):
        """Submit a ping test to the SENSE-SiteRM API"""
        self._check_action_supported("rapid-ping", **kwargs)
        kwargs["type"] = "rapid-ping"
        return self.__submit_call(**kwargs)

    def submit_pingnet(self, **kwargs):
        """Submit a ping test to the SENSE-SiteRM API"""
        self._check_action_supported("rapid-pingnet", **kwargs)
        kwargs["type"] = "rapid-pingnet"
        return self.__submit_call(**kwargs)

    def submit_arptable(self, **kwargs):
        """Submit an ARP table request to the SENSE-SiteRM API"""
        kwargs["type"] = "arp-table"
        return self.__submit_call(**kwargs)

    def submit_tcpdump(self, **kwargs):
        """Submit a tcpdump request to the SENSE-SiteRM API"""
        kwargs["type"] = "tcpdump"
        return self.__submit_call(**kwargs)

    def submit_traceroute(self, **kwargs):
        """Submit a traceroute request to the SENSE-SiteRM API"""
        kwargs["type"] = "traceroute"
        return self.__submit_call(**kwargs)

    def submit_iperfclient(self, **kwargs):
        """Submit an iperf client request to the SENSE-SiteRM API"""
        kwargs["type"] = "iperf-client"
        return self.__submit_call(**kwargs)

    def submit_iperfserver(self, **kwargs):
        """Submit an iperf server request to the SENSE-SiteRM API"""
        kwargs["type"] = "iperf-server"
        return self.__submit_call(**kwargs)

    def submit_arppush(self, **kwargs):
        """Submit an ARP push request to the SENSE-SiteRM API"""
        kwargs["type"] = "arp-push"
        return self.__submit_call(**kwargs)

    def submit_fdtserver(self, **kwargs):
        """Submit an FDT server request to the SENSE-SiteRM API"""
        kwargs["type"] = "fdt-server"
        return self.__submit_call(**kwargs)

    def submit_fdtclient(self, **kwargs):
        """Submit an FDT client request to the SENSE-SiteRM API"""
        kwargs["type"] = "fdt-client"
        return self.__submit_call(**kwargs)

    def submit_ethrserver(self, **kwargs):
        """Submit an ethr server request to the SENSE-SiteRM API"""
        kwargs["type"] = "ethr-server"
        return self.__submit_call(**kwargs)

    def submit_ethrclient(self, **kwargs):
        """Submit an ethr client request to the SENSE-SiteRM API"""
        kwargs["type"] = "ethr-client"
        return self.__submit_call(**kwargs)

    def get_debug(self, **kwargs):
        """Get debug info from SENSE-SiteRM Endpoint"""
        sitename = self.getSitename(**kwargs)
        urlparams = {}
        for key in ["details", "hostname", "state", "limit"]:
            if key in kwargs and kwargs[key]:
                urlparams[key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debug/{kwargs['id']}",
                                       **{"verb": "GET", "urlparams": urlparams})

    def get_all_debug_hostname(self, **kwargs):
        """Get all debug info from SENSE-SiteRM Endpoint"""
        sitename = self.getSitename(**kwargs)
            # build params dict instead of string
        params = {}
        for key in ["hostname", "state", "limit", "action"]:
            if key in kwargs and kwargs[key]:
                params[key] = kwargs[key]
        params["debugvar"] = "ALL"
        params["details"] = True
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debug",
                                       **{"verb": "GET", "urlparams": params})

    def delete_debug(self, **kwargs):
        """Delete debug info in SENSE-SiteRM Endpoint"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debug/{kwargs['id']}",
                                       **{"verb": "DELETE", "data": {}})

    def update_debug(self, **kwargs):
        """Update debug info in SENSE-SiteRM Endpoint"""
        sitename = self.getSitename(**kwargs)
        if "state" not in kwargs or not kwargs["state"]:
            raise Exception("State is required for update_debug")
        if "id" not in kwargs or not kwargs["id"]:
            raise Exception("ID is required for update_debug")
        data = {"id": kwargs["id"], "state": kwargs["state"]}
        if "output" in kwargs and kwargs["output"]:
            data["output"] = kwargs["output"]
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/debug/{kwargs['id']}",
                                       **{"verb": "PUT", "data": data})

    def get_dynamicranges(self, **kwargs):
        """Get dynamic ranges from SENSE-SiteRM Endpoint)"""
        sitename = self.getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/api/{sitename}/dynamicfromranges",
                                       **{"verb": "GET", "data": {}})



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Models API
"""
from sense.client.siterm.base_api import BaseApi

class ModelsApi(BaseApi):
    """Models API for SENSE-SiteRM"""
    def __init__(self):
        super().__init__()

    def get_models(self, **kwargs):
        """Get models from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['encode', 'summary', 'current', 'limit', 'rdfformat']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                        url=f"api/{sitename}/models",
                                        **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def get_model(self, **kwargs):
        """Get model from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['encode', 'summary', 'rdfformat']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                        url=f"/api/{sitename}/models/{kwargs.get('model_id')}",
                                        **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Deltas API
"""
from sense.client.siterm.base_api import BaseApi

class DeltasApi(BaseApi):
    """Deltas API for SENSE-SiteRM"""
    def __init__(self):
        super().__init__()

    def get_deltas(self, **kwargs):
        """Get deltas from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['summary', 'limit']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                        url=f"/api/{sitename}/deltas",
                                        **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def get_delta(self, **kwargs):
        """Get delta from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['summary']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                        url=f"/api/{sitename}/deltas/{kwargs.get('delta_id')}",
                                        **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def get_delta_timestates(self, **kwargs):
        """Get delta time states from the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        for key in ['limit']:
            if key in kwargs and kwargs[key]:
                kwargs.setdefault('urlparams', {})
                kwargs['urlparams'][key] = kwargs[key]
        return self.client.makeRequest(sitename=sitename,
                                        url=f"/api/{sitename}/deltas/{kwargs.get('delta_id')}/timestates",
                                        **{"verb": "GET", "data": {}, "urlparams": kwargs.get('urlparams', None)})

    def set_delta_action(self, **kwargs):
        """Set delta action in the SENSE-SiteRM API"""
        sitename = self.getSitename(**kwargs)
        if not ('delta_id' in kwargs and kwargs['delta_id']):
            raise Exception("Delta ID is required to set delta action.")
        if not ('action' in kwargs and kwargs['action']):
            raise Exception("Action is required to set delta action.")
        return self.client.makeRequest(sitename=sitename,
                                        url=f"/api/{sitename}/deltas/{kwargs['delta_id']}/actions/{kwargs['action']}",
                                        **{"verb": "PUT", "data": {}})



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DNCSchemaData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'uuid': 'str',
        'type': 'str',
        'connections': 'list[Connection]'
    }

    attribute_map = {
        'uuid': 'uuid',
        'type': 'type',
        'connections': 'connections'
    }

    def __init__(self, uuid=None, type='Multi-Point L3VPN', connections=None):  # noqa: E501
        """DNCSchemaData - a model defined in Swagger"""  # noqa: E501
        self._uuid = None
        self._type = None
        self._connections = None
        self.discriminator = None
        if uuid is not None:
            self.uuid = uuid
        self.type = type
        self.connections = connections

    @property
    def uuid(self):
        """Gets the uuid of this DNCSchemaData.  # noqa: E501

        Optional reference to instance UUID.  # noqa: E501

        :return: The uuid of this DNCSchemaData.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this DNCSchemaData.

        Optional reference to instance UUID.  # noqa: E501

        :param uuid: The uuid of this DNCSchemaData.  # noqa: E501
        :type: str
        """

        self._uuid = uuid

    @property
    def type(self):
        """Gets the type of this DNCSchemaData.  # noqa: E501

        DNC configuration mode.  # noqa: E501

        :return: The type of this DNCSchemaData.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this DNCSchemaData.

        DNC configuration mode.  # noqa: E501

        :param type: The type of this DNCSchemaData.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["Multi-Path P2P VLAN", "Multi-Point VLAN Bridge", "Multi-Point L3VPN"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def connections(self):
        """Gets the connections of this DNCSchemaData.  # noqa: E501

        The array of service connections.  # noqa: E501

        :return: The connections of this DNCSchemaData.  # noqa: E501
        :rtype: list[Connection]
        """
        return self._connections

    @connections.setter
    def connections(self, connections):
        """Sets the connections of this DNCSchemaData.

        The array of service connections.  # noqa: E501

        :param connections: The connections of this DNCSchemaData.  # noqa: E501
        :type: list[Connection]
        """
        if connections is None:
            raise ValueError("Invalid value for `connections`, must not be `None`")  # noqa: E501

        self._connections = connections

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DNCSchemaData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DNCSchemaData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
